@@ -1,4 +1,4 @@
-;; mailcrypt.el v3.5.6, mail encryption with PGP
+;; mailcrypt.el v3.5.9, mail encryption with PGP
 ;; Copyright (C) 1995  Jin Choi <jin@atype.com>
 ;;                     Patrick LoPresti <patl@lcs.mit.edu>
 ;;           (C) 1998  Len Budney <lbudney@pobox.com>
@@ -49,10 +49,11 @@
 (autoload 'mc-decrypt "mc-toplev" nil t)
 (autoload 'mc-verify "mc-toplev" nil t)
 (autoload 'mc-snarf "mc-toplev" nil t)
-(autoload 'mc-pgp-fetch-key "mc-pgp" nil t)
+(autoload 'mc-fetch-key "mc-toplev" nil t)
 (autoload 'mc-encrypt "mc-toplev" nil t)
 (autoload 'mc-sign "mc-toplev" nil t)
 (autoload 'mc-insert-public-key "mc-toplev" nil t)
+(autoload 'mc-remail "mc-toplev" nil t)
 (autoload 'mc-remailer-encrypt-for-chain "mc-remail" nil t)
 (autoload 'mc-remailer-insert-response-block "mc-remail" nil t)
 (autoload 'mc-remailer-insert-pseudonym "mc-remail" nil t)
@@ -62,7 +63,7 @@
 
 ;;{{{ Minor mode variables and functions
 
-(defvar mc-pgp-always-sign nil 
+(defvar mc-pgp-always-sign nil
   "*If t, always sign encrypted PGP messages, or never sign if 'never.")
 
 (defvar mc-read-mode nil
@@ -93,7 +94,7 @@
       (define-key mc-read-mode-map "\C-c/d" 'mc-decrypt)
       (define-key mc-read-mode-map "\C-c/v" 'mc-verify)
       (define-key mc-read-mode-map "\C-c/a" 'mc-snarf)
-      (define-key mc-read-mode-map "\C-c/k" 'mc-pgp-fetch-key)))
+      (define-key mc-read-mode-map "\C-c/k" 'mc-fetch-key)))
 
 (or mc-write-mode-map
     (progn
@@ -102,9 +103,8 @@
       (define-key mc-write-mode-map "\C-c/e" 'mc-encrypt)
       (define-key mc-write-mode-map "\C-c/s" 'mc-sign)
       (define-key mc-write-mode-map "\C-c/x" 'mc-insert-public-key)
-      (define-key mc-write-mode-map "\C-c/k" 'mc-pgp-fetch-key)
-      (define-key mc-write-mode-map "\C-c/r"
-	'mc-remailer-encrypt-for-chain)
+      (define-key mc-write-mode-map "\C-c/k" 'mc-fetch-key)
+      (define-key mc-write-mode-map "\C-c/r" 'mc-remail)
       (define-key mc-write-mode-map "\C-c/b"
 	'mc-remailer-insert-response-block)
       (define-key mc-write-mode-map "\C-c/p"
@@ -117,7 +117,7 @@
    ["Decrypt Message" mc-decrypt t]
    ["Verify Signature" mc-verify t]
    ["Snarf Keys" mc-snarf t]
-   ["Fetch Key" mc-pgp-fetch-key t]
+   ["Fetch Key" mc-fetch-key t]
    ["Forget Passphrase(s)" mc-deactivate-passwd t]))
 
 (easy-menu-define
@@ -127,8 +127,8 @@
    ["Encrypt Message" mc-encrypt t]
    ["Sign Message" mc-sign t]
    ["Insert Public Key" mc-insert-public-key t]
-   ["Fetch Key" mc-pgp-fetch-key t]
-   ["Encrypt for Remailer(s)" mc-remailer-encrypt-for-chain t]
+   ["Fetch Key" mc-fetch-key t]
+   ["Encrypt for Remailer(s)" mc-remail t]
    ["Insert Pseudonym" mc-remailer-insert-pseudonym t]
    ["Insert Response Block" mc-remailer-insert-response-block t]
    ["Forget Passphrase(s)" mc-deactivate-passwd t]))
@@ -151,13 +151,14 @@
     (setq minor-mode-alist
 	  (cons '(mc-write-mode mc-write-mode-string) minor-mode-alist)))
 
+;;;###autoload
 (defun mc-read-mode (&optional arg)
   "\nMinor mode for interfacing with cryptographic functions.
 \\<mc-read-mode-map>
 \\[mc-decrypt]\t\tDecrypt an encrypted message
 \\[mc-verify]\t\tVerify signature on a clearsigned message
 \\[mc-snarf]\t\tAdd public key(s) to keyring
-\\[mc-pgp-fetch-key]\t\tFetch a PGP key via finger or HTTP
+\\[mc-fetch-key]\t\tFetch a PGP key via finger or HTTP
 \\[mc-deactivate-passwd]\t\tForget passphrase(s)\n"
   (interactive)
   (setq mc-read-mode
@@ -167,15 +168,15 @@
   (if mc-read-mode
       (easy-menu-add mc-read-mode-menu)
     (easy-menu-remove mc-read-mode-menu)))
-	
+
 (defun mc-write-mode (&optional arg)
   "\nMinor mode for interfacing with cryptographic functions.
 \\<mc-write-mode-map>
 \\[mc-encrypt]\t\tEncrypt (and optionally sign) message
 \\[mc-sign]\t\tClearsign message
 \\[mc-insert-public-key]\t\tExtract public key from keyring and insert into message
-\\[mc-pgp-fetch-key]\t\tFetch a PGP key via finger or HTTP
-\\[mc-remailer-encrypt-for-chain]\t\tEncrypt message for remailing
+\\[mc-fetch-key]\t\tFetch a PGP key via finger or HTTP
+\\[mc-remail]\t\tEncrypt message for remailing
 \\[mc-remailer-insert-pseudonym]\t\tInsert a pseudonym (for remailing)
 \\[mc-remailer-insert-response-block]\t\tInsert a response block (for remailing)
 \\[mc-deactivate-passwd]\t\tForget passphrase(s)\n"
@@ -188,10 +189,12 @@
       (easy-menu-add mc-write-mode-menu)
     (easy-menu-remove mc-write-mode-menu)))
 
+;;;###autoload
 (defun mc-install-read-mode ()
   (interactive)
   (mc-read-mode 1))
 
+;;;###autoload
 (defun mc-install-write-mode ()
   (interactive)
   (mc-write-mode 1))
@@ -204,10 +207,18 @@
 ;;}}}
 
 ;;{{{ User variables.
-(defconst mc-version "3.5.6")
-(defvar mc-temp-directory "/tmp"
+(defconst mc-version "3.5.9")
+(defvar mc-temp-directory
+  (cond ((fboundp 'temp-directory) (temp-directory))
+	((boundp 'temporary-file-directory) temporary-file-directory)
+	("/tmp/"))
   "*Default temp directory to be used by Mailcrypt.")
-(defvar mc-default-scheme 'mc-scheme-pgp "*Default encryption scheme to use.")
+(defvar mc-default-scheme 'mc-scheme-pgp
+  "*Specifies the encryption scheme for Mailcrypt to use. Defaults
+to pgp 2.6 for backward compatibility.")
+(defvar mc-default-remailer-scheme 'mc-remailer-scheme-type1
+  "*Specifies the remailer scheme to use. Defaults to Type-1 for backward
+compatibility.")
 (defvar mc-passwd-timeout 60
   "*Time to deactivate password in seconds after a use.
 nil or 0 means deactivate immediately.  If the only timer package available
@@ -231,13 +242,13 @@ If 'never, always use a viewer instead of replacing.")
   "*List of hook functions to run immediately before signing.")
 (defvar mc-post-signature-hook nil
   "*List of hook functions to run immediately after signing.")
-(defvar mc-pre-encryption-hook nil 
+(defvar mc-pre-encryption-hook nil
   "*List of hook functions to run immediately before encrypting.")
-(defvar mc-post-encryption-hook nil 
+(defvar mc-post-encryption-hook nil
   "*List of hook functions to run after encrypting.")
-(defvar mc-pre-decryption-hook nil 
+(defvar mc-pre-decryption-hook nil
   "*List of hook functions to run immediately before decrypting.")
-(defvar mc-post-decryption-hook nil 
+(defvar mc-post-decryption-hook nil
   "*List of hook functions to run after decrypting.")
 
 (defconst mc-buffer-name "*MailCrypt*"
@@ -249,6 +260,13 @@ If 'never, always use a viewer instead of replacing.")
     (rmail-summary-mode (decrypt . mc-rmail-summary-decrypt-message)
 			(verify . mc-rmail-summary-verify-signature)
 			(snarf . mc-rmail-summary-snarf-keys))
+    (mew-draft-mode (encrypt . mc-encrypt-message)
+                    (remailer-encrypt . mc-remail-message)
+                    (sign . mc-sign-message))
+    (mew-message-mode (decrypt . mc-mew-decrypt-message))
+    (mew-summary-mode (decrypt . mc-mew-summary-decrypt-message)
+                      (verify . mc-mew-summary-verify-signature)
+                      (snarf . mc-mew-summary-snarf-keys))
     (vm-mode (decrypt . mc-vm-decrypt-message)
 	     (verify . mc-vm-verify-signature)
 	     (snarf . mc-vm-snarf-keys))
@@ -262,6 +280,7 @@ If 'never, always use a viewer instead of replacing.")
 		    (verify . mc-mh-verify-signature)
 		    (snarf . mc-mh-snarf-keys))
     (message-mode (encrypt . mc-encrypt-message)
+                  (remailer-encrypt . mc-remail-message)
                   (sign . mc-sign-message))
     (gnus-summary-mode (decrypt . mc-gnus-decrypt-message)
 		       (verify . mc-gnus-verify-signature)
@@ -270,13 +289,23 @@ If 'never, always use a viewer instead of replacing.")
 		       (verify . mc-gnus-verify-signature)
 		       (snarf . mc-gnus-snarf-keys))
     (mail-mode (encrypt . mc-encrypt-message)
+               (remailer-encrypt . mc-remail-message)
 	       (sign . mc-sign-message))
     (vm-mail-mode (encrypt . mc-encrypt-message)
+                  (remailer-encrypt . mc-remail-message)
 		  (sign . mc-sign-message))
     (mh-letter-mode (encrypt . mc-encrypt-message)
+                    (remailer-encrypt . mc-remail-message)
 		    (sign . mc-sign-message))
     (news-reply-mode (encrypt . mc-encrypt-message)
-		     (sign . mc-sign-message)))
+                     (remailer-encrypt . mc-remail-message)
+		     (sign . mc-sign-message))
+    ;; wanderlust: http://www.gohome.org/wl/
+    ;; or perhaps http://www.lab3.kuis.kyoto-u.ac.jp/~tsumura/emacs/wl.html
+    (wl-draft-mode (encrypt . mc-encrypt-message)
+                   (remailer-encrypt . mc-remail-message)
+                   (sign . mc-sign-message))
+    )
 
   "Association list (indexed by major mode) of association lists
 (indexed by operation) of functions to call for each major mode.")
@@ -289,10 +318,14 @@ If 'never, always use a viewer instead of replacing.")
 (defvar mc-passwd-cache nil "Cache for passphrases.")
 
 (defvar mc-schemes '(("pgp50" . mc-scheme-pgp50)
+                     ("pgp65" . mc-scheme-pgp65)
 		     ("pgp" . mc-scheme-pgp)
 		     ("gpg" . mc-scheme-gpg)
 		     ))
-
+(defvar mc-remailer-schemes '(("type1" . mc-remailer-scheme-type1)
+                              ("mixmaster" . mc-remailer-scheme-mixmaster)
+                              ("mixminion" . mc-remailer-scheme-mixminion)
+                              ))
 ;;}}}
 
 ;;{{{ Utility functions.
@@ -324,7 +357,7 @@ stripping initial and trailing whitespace."
 	  (setq end (string-match "[ \t\n]*\\'" str))
 	  (while (string-match regexp str beg)
 	    (setq retval
-		  (cons (substring str beg (match-beginning 0)) 
+		  (cons (substring str beg (match-beginning 0))
 			retval))
 	    (setq beg (match-end 0)))
 	  (if (not (= (length str) beg)) ; Not end
@@ -350,11 +383,11 @@ stripping initial and trailing whitespace."
 
 ;; In case I ever decide to do this right.
 ;; LRB - Thanks Pat! This helped a lot in updating mixmaster support.
-;; mc-field-name-regexp now catches precisely those email headers 
+;; mc-field-name-regexp now catches precisely those email headers
 ;; which are RFC-822 compliant.
-(defconst mc-field-name-regexp 
-  (concat 
-   "^\\([" 
+(defconst mc-field-name-regexp
+  (concat
+   "^\\(["
    (char-to-string 33) "-" (char-to-string 57)
    (char-to-string 59) "-" (char-to-string 126)
    "]*\\)"))
@@ -382,7 +415,7 @@ Optional arg NUKE, if non-nil, means eliminate all fields returned."
 	    (narrow-to-region (car bounds) (cdr bounds)))
 
 	(goto-char (point-max))
- 
+
 	(while (re-search-backward header-field-regexp nil 'move)
 	  (setq field-start (match-beginning 0))
 	  (setq field-end (match-end 0))
@@ -435,6 +468,13 @@ Email address.  ADDR-LIST may be a single string or a list of strings."
     (if msg (message-or-box "%s" msg))
     retval))
 
+(defun mc-message-sigstatus (msg &optional attention)
+  "Emit a signature status line. If ATTENTION is non-nil, be noisy about it."
+  ;; this function exists to be overridden by the unit tests
+  (if attention (ding))
+  (message-or-box "%s" msg)
+)
+
 (defun mc-process-region (beg end passwd program args parser &optional buffer)
   (let ((obuf (current-buffer))
 	(process-connection-type nil)
@@ -507,7 +547,7 @@ Return the passphrase.  If PROMPT is nil, only return value if cached."
 			      mc-passwd-timeout)))
 	   (if mc-timer (cancel-timer mc-timer))
 	   (setq mc-timer (if string-time
-			      (run-at-time string-time 
+			      (run-at-time string-time
 					   nil 'mc-deactivate-passwd)
 			    nil)))))
   (let ((cell (assoc id mc-passwd-cache))
@@ -520,6 +560,7 @@ Return the passphrase.  If PROMPT is nil, only return value if cached."
       (setq mc-passwd-cache (cons (cons id passwd) mc-passwd-cache)))
     passwd))
 
+;;;###autoload
 (defun mc-deactivate-passwd (&optional inhibit-message)
   "*Deactivate the passphrase cache."
   (interactive)
