@@ -114,9 +114,10 @@ $remailer{\"NAME\"} = \"<EMAIL ADDRESS> PROPERTIES\";
 
 PROPERTIES is a space-separated set of strings.
 
-This format is named after Raphael Levien, who maintains a list of
-active remailers.  Do \"finger rlist@publius.net\"
-for the latest copy of his list.")
+This format is named after Raphael Levien, who maintained a list of
+active remailers. Do \"finger rlist@noisebox.remailer.org\" or
+\"finger rlist@mixmaster.shinn.net\" for the latest copy of the
+remailer list.")
 
 (defvar mc-remailer-user-chains nil
   "An alist of remailer chains defined by the user.
@@ -192,12 +193,12 @@ This is a list of strings.  Completion against it will be available
 when you are prompted for your pseudonym.")
 
 (defvar mc-remailer-preserved-headers
-  '("References" "Followup-to" "In-reply-to")
+  '("References" "Followup-to" "In-reply-to" "Subject")
   "*Header fields which are preserved as hashmark headers when rewriting.
 
 This is a list of strings naming the preserved headers.  Note that
-\"Subject\", \"Newsgroups\", and \"To\" are handled specially and
-should not be included in this list.")
+\"Newsgroups\" and \"To\" are handled specially and should not be included in
+this list.")
 
 ;;}}}
 ;;{{{ Handling Levien format remailer lists
@@ -245,7 +246,7 @@ should not be included in this list.")
 	  (setq chains (cons (list remailer-name remailer) chains))))
     (goto-char (point-min))
     (if (re-search-forward "----------" nil t) ; Locate rankings at bottom
-	;; Read each word in the rankings section.  Each time we 
+	;; Read each word in the rankings section.  Each time we
 	;; hit a remailer we've identified, append it to the ranking
 	;; list.  Thus we sort remailers according to rank.
 	(while (re-search-forward "^\\([a-zA-Z0-9\\-]+\\) " nil t)
@@ -267,6 +268,7 @@ encryption."
 	    (mc-parse-levien-buffer)
 	  (bury-buffer)))))
 
+;;;###autoload
 (defun mc-reread-levien-file ()
   "Read the Levien format file specified in `mc-levien-file-name'.
 
@@ -363,7 +365,7 @@ a description of Levien file format."
      (concat "^" (regexp-quote mail-header-separator) "\n"))
     (forward-line -1)
     (cons (copy-marker (point-min)) (copy-marker (point)))))
-		
+
 (defun mc-find-colon-header (&optional insert)
   ;; Find the header with a "::" immediately after the
   ;; mail-header-separator.  Return region enclosing header.  Optional
@@ -464,12 +466,11 @@ a description of Levien file format."
 	    (if (mc-get-fields "From" colon-header)
 		"Send-To"
 	      (cond
-	       ((member "eric" props) "Anon-Send-To")
-	       ((member "cpunk" props) "Request-Remailing-To")
+	       ((member "cpunk" props) "Anon-To")
 	       (t (error "Remailer %s is not type-1" addr)))))
       (mc-replace-field to-field to colon-header)
       (mc-nuke-field "Reply-to" main-header))))
-	
+
 ;;}}}
 ;;{{{ Misc. random
 
@@ -504,6 +505,7 @@ a description of Levien file format."
       (mc-reread-levien-file))
   (append mc-remailer-internal-chains mc-remailer-user-chains))
 
+;;;###autoload
 (defun mc-remailer-insert-pseudonym ()
   "Insert pseudonym as a From field in the hash-mark header.
 
@@ -615,7 +617,7 @@ be passed to this program for rewriting.")
     (while (and rest (member "mix" (mc-remailer-properties (car rest))))
       (setq last (car rest)
 	    rest (cdr rest)))
-    
+
     ;; If recipient is not a remailer, deal with hashmark and colon
     ;; headers and get rid of them.
     (if (mc-recipient-is-remailerp)
@@ -634,7 +636,7 @@ be passed to this program for rewriting.")
 	    (goto-char (car colon-header))
 	    (forward-line -1)
 	    (delete-region (point) (+ (cdr colon-header) 1)))))
-    
+
     ;; Expand aliases and get recipients.
     (and (featurep 'mailalias)
 	 (not (featurep 'mail-abbrevs))
@@ -644,9 +646,9 @@ be passed to this program for rewriting.")
 	  (mc-cleanup-recipient-headers
 	   (mapconcat 'cdr (mc-get-fields "To" main-header t) ", ")))
     (if newsgroups
-	(setq newsgroups 
+	(setq newsgroups
 	  (mc-cleanup-recipient-headers
-	   (cdr 
+	   (cdr
 	    (assoc "Newsgroups" newsgroups)))))
     ;; Mixmaster does not support posting...
 ;    (if newsgroups
@@ -720,12 +722,6 @@ be passed to this program for rewriting.")
      (function (lambda (hook) (funcall hook remailer)))
      (mc-remailer-pre-encrypt-hooks remailer))
 
-    ;; Move "Subject" lines down.
-    (goto-char (car (mc-find-colon-header t)))
-    (mapcar
-     (function (lambda (f) (insert (car f) ":" (cdr f))))
-     (mc-get-fields "Subject" main-header t))
-
     (if pause
 	(let ((cursor-in-echo-area t))
 	  (message "SPC to encrypt for %s : " addr)
@@ -773,15 +769,17 @@ be passed to this program for rewriting.")
     (goto-char (car header))
     (if (re-search-forward (concat "^" (regexp-quote field) ":")
 			  (cdr header) t)
-	
+
 	(progn
 	  (goto-char (match-beginning 0))
 	  (error "Cannot use a %s field." field)))))
 
+;;;###autoload
 (defun mc-remailer-encrypt-for-chain (&optional pause)
   "Encrypt message for a remailer chain, prompting for chain to use.
 
-With \\[universal-argument], pause before each encryption."
+With \\[universal-argument] \\[universal-argument], pause before each
+encryption."
   (interactive "P")
   (let ((chains (mc-remailer-make-chains-alist))
 	(buffer (get-buffer-create mc-buffer-name))
@@ -808,6 +806,7 @@ With \\[universal-argument], pause before each encryption."
 ;;}}}
 ;;{{{ Response block generation
 
+;;;###autoload
 (defun mc-remailer-insert-response-block (&optional arg)
   "Insert response block at point, prompting for chain to use.
 
@@ -870,3 +869,18 @@ layer of the block before encrypting it."
     buf))
 
 ;;}}}
+
+(defun mc-remailer-type1-send (recipients start end &optional verbose)
+  ;; this function is to provide compatibility with the mc-remailer-scheme
+  ;; scheme. For type1/old-mixmaster we ignore the arguments and let the old
+  ;; functions figure them out on their own. This means
+  ;; (mc-remailer-encrypt) from non-mail buffers will only work with type2/3
+  ;; schemes. type1/old-mixmaster will get upset when it can't find the
+  ;; header-delimiter and thus can't figure out the intended recipients
+
+  (mc-remailer-encrypt-for-chain verbose))
+
+(defun mc-remailer-scheme-type1 ()
+  (list
+   (cons 'encryption-func   'mc-remailer-type1-send)
+))
