@@ -194,7 +194,7 @@ class TestCase:
         f = open(os.path.join(testcasedir, self.filename), "w")
         emit_alist(f, self.d)
         f.close()
-        
+
 
 class E_Case(TestCase):
     def __init__(self, filename, recip):
@@ -203,7 +203,23 @@ class E_Case(TestCase):
         self.encrypted_fields(recip)
     def make_crypttext(self):
         self.d['crypttext'] = "\n" + encrypt(self.recip, self.plaintext)
-        
+
+class Latin1_E_Case(E_Case):
+    # encrypt a message that contains ISO-Latin-1 characters
+    def make_plaintext(self):
+        return """
+Usted tiene que mostrar esta evidencia escrita a un oficial de polica cuando
+ste lo solicite as como a otro conductor o dueo do propiedad despus de un
+accidente vial."""
+
+class EE_Case(E_Case):
+    # encrypt to multiple recipients, some unknown
+    def make_crypttext(self):
+        self.d['crypttext'] = "\n" + \
+                              encryptor.encrypt_string(self.plaintext,
+                                                       ["unknown",
+                                                        "other",
+                                                        self.recip])
 class ES_Case(TestCase):
     def __init__(self, filename, recip, signer):
         TestCase.__init__(self, filename)
@@ -232,6 +248,14 @@ class CS_Case(TestCase):
     def make_crypttext(self):
         self.d['crypttext'] = "\n" + clearsign(self.signer, self.plaintext)
 
+class Latin1_CS_Case(CS_Case):
+    def make_plaintext(self):
+        return """
+Usted tiene que mostrar esta evidencia escrita a un oficial de polica cuando
+ste lo solicite as como a otro conductor o dueo do propiedad despus de un
+accidente vial.
+"""
+
 class SYM_Case(TestCase):
     def __init__(self, filename, passphrase):
         TestCase.__init__(self, filename)
@@ -261,6 +285,9 @@ def make_cases():
     CS_Case("CS.s2v", "trusted").process(d)
     CS_Case("CS.s3v", "untrusted").process(d)
     CS_Case("CS.s4", "unknown").process(d)
+    Latin1_CS_Case("CS.latin1.s1v", "owner1").process(d)
+    Latin1_E_Case("E.latin1.e1r", "owner1").process(d)
+    EE_Case("E.e1re3re4r", "owner1").process(d)
 
 # get keyids
 id = {}
@@ -275,12 +302,12 @@ trustmap = {
     'trusted': "MARGINAL",
     'untrusted': "UNDEFINED",
     }
-    
+
 if __name__ == "__main__":
     if not os.path.isdir(testcasedir):
         os.mkdir(testcasedir)
     make_cases()
-    
+
 
 # todo:
 #  hardcoded date in es()
